@@ -22,7 +22,9 @@ from nbxmpp.const import Role
 from nbxmpp.modules.dataforms import SimpleDataForm
 from nbxmpp.modules.security_labels import SecurityLabel
 from nbxmpp.protocol import JID
+from nbxmpp.protocol import Message
 from nbxmpp.structs import EncryptionData
+from nbxmpp.structs import HatData
 from nbxmpp.structs import MucSubject
 from nbxmpp.structs import PresenceProperties
 from nbxmpp.util import generate_id
@@ -91,6 +93,7 @@ class OutgoingMessage:
     text: dataclasses.InitVar[str | None] = None
     chatstate: str | None = None
     marker: tuple[str, str] | None = None
+    mds_id: str | None = None
     sec_label: SecurityLabel | None = None
     control: Any | None = None
     correct_id: str | None = None
@@ -144,10 +147,10 @@ class OutgoingMessage:
     def is_encrypted(self) -> bool:
         return self._encryption_data is not None
 
-    def set_stanza(self, stanza: Any) -> None:
+    def set_stanza(self, stanza: Message) -> None:
         self._stanza = stanza
 
-    def get_stanza(self) -> Any:
+    def get_stanza(self) -> Message:
         return self._stanza
 
 
@@ -158,6 +161,7 @@ class PresenceData:
     priority: int
     idle_datetime: datetime | None
     available: bool
+    hats: HatData | None
 
     @classmethod
     def from_presence(cls, properties: PresenceProperties) -> PresenceData:
@@ -170,14 +174,16 @@ class PresenceData:
                    status=properties.status,
                    priority=properties.priority,
                    idle_datetime=idle_datetime,
-                   available=properties.type.is_available)
+                   available=properties.type.is_available,
+                   hats=properties.hats)
 
 
 UNKNOWN_PRESENCE = PresenceData(show=PresenceShowExt.OFFLINE,
                                 status='',
                                 priority=0,
                                 idle_datetime=None,
-                                available=False)
+                                available=False,
+                                hats=None)
 
 
 @dataclass(frozen=True)
@@ -190,13 +196,13 @@ class MUCPresenceData:
     role: Role
     real_jid: JID | None
     occupant_id: str | None
+    hats: HatData | None
 
     @classmethod
     def from_presence(cls,
                       properties: PresenceProperties,
                       occupant_id: str | None
                       ) -> MUCPresenceData:
-
         idle_datetime = None
         if properties.idle_timestamp is not None:
             idle_datetime = convert_epoch_to_local_datetime(
@@ -209,7 +215,8 @@ class MUCPresenceData:
                    affiliation=properties.muc_user.affiliation,
                    role=properties.muc_user.role,
                    real_jid=properties.muc_user.jid,
-                   occupant_id=occupant_id)
+                   occupant_id=occupant_id,
+                   hats=properties.hats)
 
 
 UNKNOWN_MUC_PRESENCE = MUCPresenceData(show=PresenceShowExt.OFFLINE,
@@ -219,7 +226,8 @@ UNKNOWN_MUC_PRESENCE = MUCPresenceData(show=PresenceShowExt.OFFLINE,
                                        affiliation=Affiliation.NONE,
                                        role=Role.NONE,
                                        real_jid=None,
-                                       occupant_id=None)
+                                       occupant_id=None,
+                                       hats=None)
 
 
 class VariantMixin:
@@ -238,7 +246,7 @@ class VariantMixin:
         if 'Optional' in field_type:
             variant_str = 'm'
 
-        if 'str' in field_type:
+        if 'str' in field_type or 'Literal' in field_type:
             return str, f'{variant_str}s'
 
         if 'int' in field_type:
@@ -263,7 +271,9 @@ class VariantMixin:
                 continue
 
             if not isinstance(value, field_t):
-                raise ValueError(f'invalid type: {value} is not a {field_t}')
+                raise ValueError(
+                    f'invalid type: {value} type {type(value)} is not a {field_t}'
+                )
 
             conversion_func = self._type_to_variant_funcs.get(field_t)
             if conversion_func is not None:
